library(truncnorm)
library(survival)
library(cmprsk)
# library(miceadds)
library(RColorBrewer)
cols<-brewer.pal(n=12,name='Paired')
cols<-cols[c(2,4,6,8,10,12)]

## Simulate Scan Visiting Matrics for PFS
sim_vmtx<-function(size,scheduled_visits,window,Pr_usv){
  vplanned<-length(scheduled_visits) # maximal scheduled visit per patient
  vuplanned<-round(Pr_usv*vplanned,0) # maximal total visit per patient
  
  mat_visit <- matrix( rep(NA,size*(vplanned+vuplanned)), nrow= size) # matrix initialization
  for(i in 1:size){
    visitwindow<-round(rtruncnorm(vplanned, a=-7*window, b=7*window, mean = 0, sd = 4),0)/7 
    actual_visit0<-scheduled_visits+visitwindow
    unscheduled_visits<- runif(vuplanned, min(actual_visit0),max(actual_visit0))
    actual_visit<-round(sort(c(actual_visit0,unscheduled_visits)),2)
    mat_visit[i,]<-actual_visit
  }
  mat_month<-mat_visit*7/365.25*12
  return(mat_month)
}
# visit_mtx<-round(sim_vmtx(size = 20, scheduled_visits=c(seq(6, 48, by=6),seq(48, 96, by=8)), window=1, Pr_usv=0.1 ),3)
# > visit_mtx
# [,1] [,2] [,3] [,4] [,5] [,6] [,7]  [,8]  [,9] [,10] [,11] [,12] [,13] [,14] [,15] [,16] [,17]
# [1,] 1.31 2.63 4.11 5.49 7.13 8.18 9.63 11.04 11.24 12.29 12.72 14.65 16.72 18.56 19.86 20.24 21.98
# [2,] 1.45 2.76 3.76 4.11 5.49 6.97 8.12  9.46 10.94 11.01 12.85 12.98 14.75 16.46 18.50 20.37 21.91
# [3,] 1.45 2.79 4.27 5.59 6.87 8.28 9.50 11.11 11.14 11.83 12.88 13.13 14.69 16.53 18.53 20.01 21.91
# [4,] 1.41 2.76 4.21 5.08 5.72 5.83 6.87  8.28  9.79 10.91 11.11 12.81 14.65 16.49 18.53 20.34 22.08
# [5,] 1.45 2.86 4.14 5.42 5.67 6.77 8.21  9.50 11.07 11.07 12.85 14.33 14.56 16.49 18.20 20.11 22.14
# [6,] 1.31 2.73 4.04 5.42 6.93 8.25 9.63 10.88 11.04 12.75 14.65 15.46 16.72 18.37 20.40 20.42 21.85
# [7,] 1.54 2.76 4.07 4.94 5.45 6.83 8.18  9.56 10.91 10.97 12.68 14.62 16.69 18.20 20.14 21.59 22.18
# [8,] 1.31 2.76 3.94 5.42 6.97 8.28 9.69 10.84 11.17 12.68 14.69 16.22 16.40 17.65 18.40 20.21 22.18
# [9,] 1.15 2.66 3.91 4.07 4.29 5.52 6.80  8.15  9.53 10.94 11.14 12.85 14.75 16.69 18.33 20.40 22.11
# [10,] 1.58 2.76 4.11 4.57 5.65 7.03 8.38  9.33  9.56 10.81 11.04 12.95 14.88 16.43 18.20 20.24 22.08

onesidep.survdiff<-function(time,censor,trt){
  lr <- survdiff(Surv(time,censor) ~ trt)
  csp <- pchisq(lr$chisq, length(unique(trt)) - 1, lower.tail = FALSE)
  pval <- ifelse((lr$obs[2] - lr$exp[2]) <= 0, csp/2, 1 - csp/2)  
  return(pval)
}

## Simulate OS date: Poseidon
sim.data<-function(size,mOSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration){
  # size: size of two arm
  # mOSc: median of control
  # IAevent: target number of event in Interim
  # FAevent: target number of event in final
  # rr: randomzation ratio
  # PanRate1: Pandemic impact rate in control group before adjusting for competing risk
  # RanRate2: Pandemic impact rate in treatment group before adjusting for competing risk
  # IA2PanTime: If there is interim after pandemic, when is the time gap between the onset of Pandemic and the interim
  # Panduration: duration of pandemic
  
  
  # size=50;mOSc=11.5;HR=0.7;accrual.duration=22;accrual.weight=2;IAevent=20; FAevent=45;rr=1;PanN=4;PanScenario=1;IA2PanTime=2;Panduration=3
  
  lambda_ctrl = log(2)/mOSc
  lambda_act = lambda_ctrl*HR
  entryt <- runif(size, 0, 1)
  entryt <- (((accrual.duration)^accrual.weight)*entryt)^(1/accrual.weight)
  
  t1<-rexp(size/(rr+1),lambda_ctrl)
  t2<-rexp(rr*size/(rr+1),lambda_act)
  
  fultime<-c(t1,t2)
  trt<-c(rep(0,size/(rr+1)),rep(1,rr*size/(rr+1)))
  caltime0<-entryt+fultime
  
  SUBJID<-seq(1,size,1)
  data<-as.data.frame(cbind(SUBJID,trt,fultime,entryt,caltime0)) # 0 control 1 treatment
  
  PanFUS<-quantile(caltime0,FAevent/size)-FA2PanTime
  PanFUF<-PanFUS + Panduration
  
  ## Generate Pandemic Data
  data$caltime1<-data$caltime0
  
  if (PanScenario == 1){
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    for (i in 1:PanN){
      ind<-sample(1:dim(data)[1], 1, replace=F)
      # while (data[ind,]$caltime1<CRtime[i] | ind%in%pos){
      while (data[ind,]$caltime1<CRtime[i] | data[ind,]$entryt>CRtime[i]  | ind%in%pos){
        ind<-sample(1:dim(data)[1], 1, replace=F)
      }
      pos[i]<-ind
      data[ind,]$caltime1<-CRtime[i]
    }
  } else if (PanScenario==2){
    data_0<-data[1:round(size/(rr+1),0),]
    data_1<-data[(round(size/(rr+1),0)+1):size,]
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    for (i in 1:PanN){
      ind<-sample(1:dim(data_1)[1], 1, replace=F)
      # while (data_1[ind,]$caltime1<CRtime[i] | ind%in%pos){
      while (data_1[ind,]$caltime1<CRtime[i] | data_1[ind,]$entryt>CRtime[i] | ind%in%pos){
        ind<-sample(1:dim(data_1)[1], 1, replace=F)
      }
      pos[i]<-ind
      data_1[ind,]$caltime1<-CRtime[i]
    }
    data<-rbind(data_0,data_1)
  } else if (PanScenario==3){
    data_0<-data[1:round(size/(rr+1),0),]
    data_1<-data[(round(size/(rr+1),0)+1):size,]
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    for (i in 1:PanN){
      ind<-sample(1:dim(data_0)[1], 1, replace=F)
      # while (data_0[ind,]$caltime1<CRtime[i] | ind%in%pos){
      while (data_0[ind,]$caltime1<CRtime[i] | data_0[ind,]$entryt>CRtime[i] | ind%in%pos){
        ind<-sample(1:dim(data_0)[1], 1, replace=F)
      }
      pos[i]<-ind
      data_0[ind,]$caltime1<-CRtime[i]
    }
    data<-rbind(data_0,data_1)
  }
  
  data$covidind[data$caltime0==data$caltime1]<-0
  data$covidind[data$caltime0!=data$caltime1]<-1
  
  ## Scenario 0: No COVID19, this is the reference data to evaluate the impact of COVID19
  dcoIA0_cutoff<-quantile(data$caltime0,IAevent/size)
  data$dcoIA0caltime<-pmin(dcoIA0_cutoff,data$caltime0)
  data$dcoIA0time <- data$dcoIA0caltime - data$entryt # Scenario 0 interim survival time
  data$dcoIA0ind<-as.numeric(data$caltime0<=data$dcoIA0caltime) # Scenario 0 interim censoring indicator
  
  dcoFA0_cutoff<-quantile(data$caltime0,FAevent/size)
  data$dcoFA0caltime<-pmin(dcoFA0_cutoff,data$caltime0)
  data$dcoFA0time <- data$dcoFA0caltime - data$entryt # Scenario 0 final survival time
  data$dcoFA0ind <- as.numeric(data$caltime0 <= data$dcoFA0caltime) # Scenario 0 final censoring indicator
  
  ## Scenario 1: ITT approach: Include all deaths due to any causes with COVID19 pandemic
  dcoIA1_cutoff<-quantile(data$caltime1,IAevent/size)
  data$dcoIA1caltime<-pmin(dcoIA1_cutoff,data$caltime1)
  data$dcoIA1time <- data$dcoIA1caltime - data$entryt
  data$dcoIA1ind<-as.numeric(data$caltime1<=data$dcoIA1caltime)
  
  dcoFA1_cutoff<-quantile(data$caltime1,FAevent/size)
  data$dcoFA1caltime<-pmin(dcoFA1_cutoff,data$caltime1)
  data$dcoFA1time <- data$dcoFA1caltime - data$entryt
  data$dcoFA1ind<-as.numeric(data$caltime1<=data$dcoFA1caltime)
  
  ## Scenario 2: mITT population: exclude patients with COVID-related death from the analysis
  ## data1_mITT <- subset(data1,covidind!=1)
  
  ## Scenario 3: Censor patients at the time of COVID death and target the number of non-COVID deaths and assess the impact on the updated time to events(DCO)
  # data$covidind<-as.numeric(data$pp & (data$CRtime<data$caltime0))
  dcoIA2_cutoff<-quantile(data[which(data$covidind!=1),]$caltime1,IAevent/(size-sum(data$covidind)))
  data$dcoIA2caltime<-pmin(dcoIA2_cutoff,data$caltime1)
  data$dcoIA2time <- data$dcoIA2caltime - data$entryt
  data$dcoIA2ind<-as.numeric(data$caltime1<=data$dcoIA2caltime & data$covidind!=1)
  
  dcoFA2_cutoff<-quantile(data[which(data$covidind!=1),]$caltime1,FAevent/(size-sum(data$covidind)))
  data$dcoFA2caltime<-pmin(dcoFA2_cutoff,data$caltime1)
  data$dcoFA2time <- data$dcoFA2caltime - data$entryt
  data$dcoFA2ind<-as.numeric(data$caltime1<=data$dcoFA2caltime & data$covidind!=1)
  
  ## Scenario 4: Cenpatients as the time of COVID19 death but still use the original DCO
  data$dcoIA3ind<-as.numeric(data$caltime1<=data$dcoIA1caltime & data$covidind!=1)
  data$dcoFA3ind<-as.numeric(data$caltime1<=data$dcoFA1caltime & data$covidind!=1)
  
  ## Scenario 5: Competing Risk
  data$dcoIA4ind[data$covidind==1 & data$dcoIA1ind==1]<-2
  data$dcoIA4ind[data$covidind==0 & data$dcoIA1ind==1]<-1
  data$dcoIA4ind[data$covidind==1 & data$dcoIA1ind==0]<-0
  data$dcoIA4ind[data$covidind==0 & data$dcoIA1ind==0]<-0
  
  data$dcoFA4ind[data$covidind==1 & data$dcoFA1ind==1]<-2
  data$dcoFA4ind[data$covidind==0 & data$dcoFA1ind==1]<-1
  data$dcoFA4ind[data$covidind==1 & data$dcoFA1ind==0]<-0
  data$dcoFA4ind[data$covidind==0 & data$dcoFA1ind==0]<-0
  
  
  names(data)<-c('SUBJID','trt','fultime','entryt','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                 'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
  data<-as.data.frame(data)
  return(data)
  
  ## The Scenario 1-4 are setup in data preparation and use regular logrank test and Cox model
  ## The Scenario 5 are setup in data preparation and use competing risk model
}

## Simulate PFS date: PACIFIC4
sim.PFSdata<-function(size,mPFSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration, scheduled_visits, window, Pr_usv){
  # size: size of two arm
  # mPFSc: median of control
  # IAevent: target number of event in Interim
  # FAevent: target number of event in final
  # rr: randomzation ratio
  # PanRate1: Pandemic impact rate in control group before adjusting for competing risk
  # RanRate2: Pandemic impact rate in treatment group before adjusting for competing risk
  # IA2PanTime: If there is interim after pandemic, when is the time gap between the onset of Pandemic and the interim
  # Panduration: duration of pandemic
  # scheduled_visits: scheduled visits in weeks, will be transfer to month by sim_vmtx function
  # window: visit window
  # Pr_usv: prob of unscheduled visit
  
  # size=40;mPFSc=6;HR=0.7;accrual.duration=22;accrual.weight=2;IAevent=10; FAevent=18;rr=1;PanN=2;PanScenario=1;IA2PanTime=2;Panduration=3
  # scheduled_visits=c(seq(6, 48, by=6),seq(48, 96, by=8)); window=1; Pr_usv=0.1
  size=530;mPFSc=26;HR=1;accrual.duration=29;accrual.weight=2;IAevent=233; FAevent=311;rr=1;PanN=15;PanScenario=1;FA2PanTime=45;Panduration=3;scheduled_visits=c(seq(8, 116, by=12),seq(132, 148, by=16),seq(174, 312, by=26)); window=1; Pr_usv=0.1
  
  lambda_ctrl = log(2)/mPFSc
  lambda_act = lambda_ctrl*HR
  entryt <- runif(size, 0, 1)
  entryt <- (((accrual.duration)^accrual.weight)*entryt)^(1/accrual.weight)
  
  t1<-rexp(size/(rr+1),lambda_ctrl)
  t2<-rexp(rr*size/(rr+1),lambda_act)
  
  fultime<-c(t1,t2)
  trt<-c(rep(0,size/(rr+1)),rep(1,rr*size/(rr+1)))
  
  SUBJID<-seq(1,size,1)
  data<-as.data.frame(cbind(SUBJID,trt,fultime,entryt)) # 0 control 1 treatment
  
  visit_mtx<-as.data.frame(round(sim_vmtx(size = size, scheduled_visits=scheduled_visits, window=window, Pr_usv=Pr_usv), 4))
  
  # data<-cbind(data,visit_mtx)
  visit_mtx<-cbind(visit_mtx,fultime)
  pos<-apply(fultime>visit_mtx,1,sum)+1
  data$pfs<-rep(NA,size)
  for(i in 1:size) data$pfs[i]<-visit_mtx[i,pos[i]] 
  
  data$caltime0<-data$entryt+data$pfs
  
  PanFUS<-quantile(data$caltime0,FAevent/size)-FA2PanTime
  PanFUF<-PanFUS + Panduration
  
  ## Generate Pandemic Data
  data$caltime1<-data$caltime0
  if (PanScenario == 1){ 
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    dataA<-subset(data,caltime1<min(CRtime) | entryt>max(CRtime))  ## exclude patient die before pandemic start or enrolled after pandemic ended
    dataB<-subset(data,caltime1>=min(CRtime) & entryt<=max(CRtime))
    
    for (i in 1:PanN){
      ind<-sample(1:dim(dataB)[1], 1, replace=F)
      while (dataB[ind,]$caltime1<CRtime[i] | dataB[ind,]$entryt>CRtime[i]  | ind%in%pos){  ## if the candidate patient 'die' before the specific COVID death date, or enrolled after COVID death date, 
                                                                                            ## or he is already been select as COVID death, resample again 
        ind<-sample(1:dim(dataB)[1], 1, replace=F)
      }
      pos[i]<-ind
      dataB[ind,]$caltime1<-CRtime[i]
    }
    data<-rbind(dataA,dataB)
  } else if (PanScenario==2){ 
    data_0<-data[1:round(size/(rr+1),0),]
    data_1<-data[(round(size/(rr+1),0)+1):size,]
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    dataA<-subset(data_1,caltime1<min(CRtime) | entryt>max(CRtime))
    dataB<-subset(data_1,caltime1>=min(CRtime) & entryt<=max(CRtime))
    for (i in 1:PanN){
      ind<-sample(1:dim(dataB)[1], 1, replace=F)
      # while (data_1[ind,]$caltime1<CRtime[i] | ind%in%pos){
      while (dataB[ind,]$caltime1<CRtime[i] | dataB[ind,]$entryt>CRtime[i] | ind%in%pos){  
        ind<-sample(1:dim(dataB)[1], 1, replace=F)
      }
      pos[i]<-ind
      dataB[ind,]$caltime1<-CRtime[i]
    }
    data<-rbind(data_0,dataA,dataB)
  } else if (PanScenario==3){ 
    data_0<-data[1:round(size/(rr+1),0),]
    data_1<-data[(round(size/(rr+1),0)+1):size,]
    CRtime<-runif(PanN, PanFUS,PanFUF);pos<-NA
    dataA<-subset(data_0,caltime1<min(CRtime) | entryt>max(CRtime))
    dataB<-subset(data_0,caltime1>=min(CRtime) & entryt<=max(CRtime))
    for (i in 1:PanN){
      ind<-sample(1:dim(dataB)[1], 1, replace=F)
      # while (data_0[ind,]$caltime1<CRtime[i] | ind%in%pos){
      while (dataB[ind,]$caltime1<CRtime[i] | dataB[ind,]$entryt>CRtime[i] | ind%in%pos){
        ind<-sample(1:dim(dataB)[1], 1, replace=F)
      }
      pos[i]<-ind
      dataB[ind,]$caltime1<-CRtime[i]
    }
    data<-rbind(dataA,dataB,data_1)
  }
  
  data$covidind[data$caltime0==data$caltime1]<-0
  data$covidind[data$caltime0!=data$caltime1]<-1
  
  ## Scenario 0: No COVID19, this is the reference data to evaluate the impact of COVID19
  dcoIA0_cutoff<-quantile(data$caltime0,IAevent/size) 
  data$dcoIA0caltime<-pmin(dcoIA0_cutoff,data$caltime0)  
  data$dcoIA0time <- data$dcoIA0caltime - data$entryt # Scenario 0 interim survival time
  data$dcoIA0ind<-as.numeric(data$caltime0<=data$dcoIA0caltime) # Scenario 0 interim censoring indicator
  
  dcoFA0_cutoff<-quantile(data$caltime0,FAevent/size)
  data$dcoFA0caltime<-pmin(dcoFA0_cutoff,data$caltime0)  
  data$dcoFA0time <- data$dcoFA0caltime - data$entryt # Scenario 0 final survival time
  data$dcoFA0ind <- as.numeric(data$caltime0 <= data$dcoFA0caltime) # Scenario 0 final censoring indicator
  
  ## Scenario 1: ITT approach: Include all deaths due to any causes with COVID19 pandemic
  dcoIA1_cutoff<-quantile(data$caltime1,IAevent/size) 
  data$dcoIA1caltime<-pmin(dcoIA1_cutoff,data$caltime1)  
  data$dcoIA1time <- data$dcoIA1caltime - data$entryt
  data$dcoIA1ind<-as.numeric(data$caltime1<=data$dcoIA1caltime)
  
  dcoFA1_cutoff<-quantile(data$caltime1,FAevent/size)
  data$dcoFA1caltime<-pmin(dcoFA1_cutoff,data$caltime1)  
  data$dcoFA1time <- data$dcoFA1caltime - data$entryt
  data$dcoFA1ind<-as.numeric(data$caltime1<=data$dcoFA1caltime)
  
  ## Scenario 2: mITT population: exclude patients with COVID-related death from the analysis
  ## data1_mITT <- subset(data1,covidind!=1)  
  
  ## Scenario 3: Censor patients at the time of COVID death and target the number of non-COVID deaths and assess the impact on the updated time to events(DCO)
  
  dcoIA2_cutoff<-quantile(data[which(data$covidind!=1),]$caltime1,IAevent/(size-sum(data$covidind)))
  data$dcoIA2caltime<-pmin(dcoIA2_cutoff,data$caltime1)  
  data$dcoIA2time <- data$dcoIA2caltime - data$entryt
  data$dcoIA2ind<-as.numeric(data$caltime1<=data$dcoIA2caltime & data$covidind!=1)
  
  dcoFA2_cutoff<-quantile(data[which(data$covidind!=1),]$caltime1,FAevent/(size-sum(data$covidind)))
  data$dcoFA2caltime<-pmin(dcoFA2_cutoff,data$caltime1)  
  data$dcoFA2time <- data$dcoFA2caltime - data$entryt
  data$dcoFA2ind<-as.numeric(data$caltime1<=data$dcoFA2caltime & data$covidind!=1)
  
  ## Scenario 4: Cenpatients as the time of COVID19 death but still use the original DCO
  data$dcoIA3ind<-as.numeric(data$caltime1<=data$dcoIA1caltime & data$covidind!=1)
  data$dcoFA3ind<-as.numeric(data$caltime1<=data$dcoFA1caltime & data$covidind!=1)
  
  ## Scenario 5: Competing Risk
  data$dcoIA4ind[data$covidind==1 & data$dcoIA1ind==1]<-2 # die before DCO, covid death
  data$dcoIA4ind[data$covidind==0 & data$dcoIA1ind==1]<-1 # die before DCO, non covid death
  data$dcoIA4ind[data$covidind==1 & data$dcoIA1ind==0]<-0 # no death by DCO, covid death
  data$dcoIA4ind[data$covidind==0 & data$dcoIA1ind==0]<-0 # no death by DCO
  
  data$dcoFA4ind[data$covidind==1 & data$dcoFA1ind==1]<-2 
  data$dcoFA4ind[data$covidind==0 & data$dcoFA1ind==1]<-1
  data$dcoFA4ind[data$covidind==1 & data$dcoFA1ind==0]<-0
  data$dcoFA4ind[data$covidind==0 & data$dcoFA1ind==0]<-0
  
  
  names(data)<-c('SUBJID','trt','fultime','entryt','pfs','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                 'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
  data<-as.data.frame(data)
  return(data)
  
  ## The Scenario 1-4 are setup in data preparation and use regular logrank test and Cox model
  ## The Scenario 5 are setup in data preparation and use competing risk model
}

simOS<-function(size,mOSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration,simu){
  tt <- simu*size*29
  data<- array(1:tt, dim=c(size,29,simu))
  for (i in 1:simu){
    data[,,i]<-as.matrix(sim.data(size,mOSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration))
    if(i %in% seq(1000,simu,1000)) cat(i)
  }
  return(data)
}

simPFS<-function(size,mPFSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration, scheduled_visits, window, Pr_usv,simu){
  tt <- simu*size*30
  data<- array(1:tt, dim=c(size,30,simu))
  for (i in 1:simu){
    data[,,i]<-as.matrix(sim.PFSdata(size,mPFSc,HR,accrual.duration,accrual.weight,IAevent,FAevent,rr,PanN,PanScenario,FA2PanTime,Panduration, scheduled_visits, window, Pr_usv))
    if(i %in% seq(100,simu,100)) cat(i)
  }
  return(data)
}


## The evaluation is based on one sided evaluation, if two sided alpha, alpha need be devided by two.
## only one IA allowed for evaluation 
simPower<-function(data,alphaFA,size,hr.cv.FA, FAevent, simu,includeIA=F,alphaIA,hr.cv.IA,isOS=T){
  
  data<-miceadds::load.Rdata2( filename=paste0(data,".Rdata"))  
  ind0.1<-ind0.2<-ind1.1<-ind1.2<-ind2.1<-ind2.2<-ind3.1<-ind3.2<-ind4.1<-ind4.2<-ind5.1<-ind5.2<-rep(0,simu)
  logHR0.1<-logHR0.2<-logHR1.1<-logHR1.2<-logHR2.1<-logHR2.2<-logHR3.1<-logHR3.2<-logHR4.1<-logHR4.2<-logHR5.1<-logHR5.2<-rep(NA,simu)
  time.0<-time.1<-time.2<-rep(NA,simu)
  
  if (includeIA & isOS){ # OS with IA
    for (i in 1:simu){
      data1<-as.data.frame(data[,,i])
      colnames(data1)<-c('SUBJID','trt','fultime','entryt','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                         'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
      data1_mITT <- subset(data1,covidind!=1)
      covmatrix<- matrix(as.numeric(data1$trt == 1));colnames(covmatrix) <- "trt" # covariate treated as matrix, required in crr.
      
      os_cp0.1<-coxph(Surv(dcoIA0time,dcoIA0ind)~trt,ties="efron" ,data=data1); #IA, No COVID
      os_cp0.2<-coxph(Surv(dcoFA0time,dcoFA0ind)~trt,ties="efron" ,data=data1); #FA, No COVID
      os_cp1.1<-coxph(Surv(dcoIA1time,dcoIA1ind)~trt,ties="efron" ,data=data1); #IA, COVID ITT
      os_cp1.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1); #FA, COVID ITT
      os_cp2.1<-coxph(Surv(dcoIA1time,dcoIA1ind)~trt,ties="efron" ,data=data1_mITT); #IA, COVID mITT
      os_cp2.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1_mITT); #FA, COVID mITT
      os_cp3.1<-coxph(Surv(dcoIA2time,dcoIA2ind)~trt,ties="efron" ,data=data1); #IA, Censor COVID with updated DCO
      os_cp3.2<-coxph(Surv(dcoFA2time,dcoFA2ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with updated DCO
      os_cp4.1<-coxph(Surv(dcoIA1time,dcoIA3ind)~trt,ties="efron" ,data=data1); #IA, Censor COVID with original DCO
      os_cp4.2<-coxph(Surv(dcoFA1time,dcoFA3ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with original DCO
      os_cp5.1<-crr(ftime = data1$dcoIA1time, fstatus = data1$dcoIA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #IA, Competing Risk
      os_cp5.2<-crr(ftime = data1$dcoFA1time, fstatus = data1$dcoFA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #FA, Competing Risk
      
      lrpval0.1<-onesidep.survdiff(data1$dcoIA0time,data1$dcoIA0ind,data1$trt)
      lrpval0.2<-onesidep.survdiff(data1$dcoFA0time,data1$dcoFA0ind,data1$trt)
      lrpval1.1<-onesidep.survdiff(data1$dcoIA1time,data1$dcoIA1ind,data1$trt)
      lrpval1.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA1ind,data1$trt)
      lrpval2.1<-onesidep.survdiff(data1_mITT$dcoIA1time,data1_mITT$dcoIA1ind,data1_mITT$trt)
      lrpval2.2<-onesidep.survdiff(data1_mITT$dcoFA1time,data1_mITT$dcoFA1ind,data1_mITT$trt)
      lrpval3.1<-onesidep.survdiff(data1$dcoIA2time,data1$dcoIA2ind,data1$trt)
      lrpval3.2<-onesidep.survdiff(data1$dcoFA2time,data1$dcoFA2ind,data1$trt)      
      lrpval4.1<-onesidep.survdiff(data1$dcoIA1time,data1$dcoIA3ind,data1$trt)
      lrpval4.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA3ind,data1$trt)
      
      csp5.1<-pchisq(summary(os_cp5.1)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.1 <- ifelse( os_cp5.1$coef <= 0, csp5.1/2, 1 - csp5.1/2)  
      csp5.2<-pchisq(summary(os_cp5.2)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.2 <- ifelse( os_cp5.2$coef <= 0, csp5.2/2, 1 - csp5.2/2)  
      
      ind0.1[i]<-as.numeric(lrpval0.1<alphaIA)
      ind0.2[i]<-as.numeric(lrpval0.2<alphaFA)
      ind1.1[i]<-as.numeric(lrpval1.1<alphaIA)
      ind1.2[i]<-as.numeric(lrpval1.2<alphaFA)
      ind2.1[i]<-as.numeric(lrpval2.1<alphaIA)
      ind2.2[i]<-as.numeric(lrpval2.2<alphaFA)
      ind3.1[i]<-as.numeric(lrpval3.1<alphaIA)
      ind3.2[i]<-as.numeric(lrpval3.2<alphaFA)
      ind4.1[i]<-as.numeric(lrpval4.1<alphaIA)
      ind4.2[i]<-as.numeric(lrpval4.2<alphaFA)
      ind5.1[i]<-as.numeric(plrtpval5.1<alphaIA)
      ind5.2[i]<-as.numeric(plrtpval5.2<alphaFA)      
      
      logHR0.1[i]<-summary(os_cp0.1)$coefficients[1]
      logHR0.2[i]<-summary(os_cp0.2)$coefficients[1]
      logHR1.1[i]<-summary(os_cp1.1)$coefficients[1]
      logHR1.2[i]<-summary(os_cp1.2)$coefficients[1]
      logHR2.1[i]<-summary(os_cp2.1)$coefficients[1]
      logHR2.2[i]<-summary(os_cp2.2)$coefficients[1]
      logHR3.1[i]<-summary(os_cp3.1)$coefficients[1]
      logHR3.2[i]<-summary(os_cp3.2)$coefficients[1]
      logHR4.1[i]<-summary(os_cp4.1)$coefficients[1]
      logHR4.2[i]<-summary(os_cp4.2)$coefficients[1]
      logHR5.1[i]<-summary(os_cp5.1)$coef[1]
      logHR5.2[i]<-summary(os_cp5.2)$coef[1]
      
      time.0[i]<-sort(data1[data1$dcoFA0ind==1,]$dcoFA0caltime)[FAevent] #No COVID19, TIME for FA
      time.1[i]<-sort(data1[data1$dcoFA1ind==1,]$dcoFA1caltime)[FAevent] #COVID19, TIME for FA
      time.2[i]<-sort(data1[data1$dcoFA2ind==1,]$dcoFA2caltime)[FAevent] #COVID19, TIME for FA with censoring approach
      if(i %in% seq(1000,simu,1000)) cat(i)
    }
    power0.1<-mean(ind0.1);power0.2<-mean(ind0.2);power1.1<-mean(ind1.1);power1.2<-mean(ind1.2);
    power2.1<-mean(ind2.1);power2.2<-mean(ind2.2);power3.1<-mean(ind3.1);power3.2<-mean(ind3.2);
    power4.1<-mean(ind4.1);power4.2<-mean(ind4.2);power5.1<-mean(ind5.1);power5.2<-mean(ind5.2);
    
    mHR0.1<-exp(mean(logHR0.1));mHR0.2<-exp(mean(logHR0.2));mHR1.1<-exp(mean(logHR1.1));mHR1.2<-exp(mean(logHR1.2));
    mHR2.1<-exp(mean(logHR2.1));mHR2.2<-exp(mean(logHR2.2));mHR3.1<-exp(mean(logHR3.1));mHR3.2<-exp(mean(logHR3.2));
    mHR4.1<-exp(mean(logHR4.1));mHR4.2<-exp(mean(logHR4.2));mHR5.1<-exp(mean(logHR5.1));mHR5.2<-exp(mean(logHR5.2));
    
    crHR0.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR0.1)))
    crHR0.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR0.2)))
    crHR1.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR1.1)))
    crHR1.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR1.2)))
    crHR2.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR2.1)))
    crHR2.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR2.2)))
    crHR3.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR3.1)))
    crHR3.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR3.2)))
    crHR4.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR4.1)))
    crHR4.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR4.2)))
    crHR5.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR5.1)))
    crHR5.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR5.2)))
    
    time0<-mean(time.0,na.rm=T)
    time1<-mean(time.1,na.rm=T)
    time2<-mean(time.2,na.rm=T)
    
    powerIA<-c(power0.1,power1.1,power2.1,power3.1,power4.1,power5.1)
    powerFA<-c(power0.2,power1.2,power2.2,power3.2,power4.2,power5.2)
    mHRIA  <-c(mHR0.1,mHR1.1,mHR2.1,mHR3.1,mHR4.1,mHR5.1)
    mHRFA  <-c(mHR0.2,mHR1.2,mHR2.2,mHR3.2,mHR4.2,mHR5.2)
    crHRIA <-c(crHR0.1,crHR1.1,crHR2.1,crHR3.1,crHR4.1,crHR5.1)
    crHRFA <-c(crHR0.2,crHR1.2,crHR2.2,crHR3.2,crHR4.2,crHR5.2)
    time <-c(time0,time1,time2)
    result<-list(round(powerIA,4),round(powerFA,4),round(mHRIA,4),round(mHRFA,4),round(crHRIA,4),round(crHRFA,4),round(time,4))
    names(result)<-c('PowerIA','PowerFA','HRIA','HRFA','CVHRIA','CVHRFA','Time')
  }
  
  else if (includeIA & !isOS){ # PFS with IA
    
    for (i in 1:simu){
      data1<-as.data.frame(data[,,i])
      colnames(data1)<-c('SUBJID','trt','fultime','entryt','pfs','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                         'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
      data1_mITT <- subset(data1,covidind!=1)
      covmatrix<- matrix(as.numeric(data1$trt == 1));colnames(covmatrix) <- "trt" # covariate treated as matrix, required in crr.
      
      os_cp0.1<-coxph(Surv(dcoIA0time,dcoIA0ind)~trt,ties="efron" ,data=data1); #IA, No COVID
      os_cp0.2<-coxph(Surv(dcoFA0time,dcoFA0ind)~trt,ties="efron" ,data=data1); #FA, No COVID
      os_cp1.1<-coxph(Surv(dcoIA1time,dcoIA1ind)~trt,ties="efron" ,data=data1); #IA, COVID ITT
      os_cp1.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1); #FA, COVID ITT
      os_cp2.1<-coxph(Surv(dcoIA1time,dcoIA1ind)~trt,ties="efron" ,data=data1_mITT); #IA, COVID mITT
      os_cp2.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1_mITT); #FA, COVID mITT
      os_cp3.1<-coxph(Surv(dcoIA2time,dcoIA2ind)~trt,ties="efron" ,data=data1); #IA, Censor COVID with updated DCO
      os_cp3.2<-coxph(Surv(dcoFA2time,dcoFA2ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with updated DCO
      os_cp4.1<-coxph(Surv(dcoIA1time,dcoIA3ind)~trt,ties="efron" ,data=data1); #IA, Censor COVID with original DCO
      os_cp4.2<-coxph(Surv(dcoFA1time,dcoFA3ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with original DCO
      os_cp5.1<-crr(ftime = data1$dcoIA1time, fstatus = data1$dcoIA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #IA, Competing Risk
      os_cp5.2<-crr(ftime = data1$dcoFA1time, fstatus = data1$dcoFA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #FA, Competing Risk
      
      lrpval0.1<-onesidep.survdiff(data1$dcoIA0time,data1$dcoIA0ind,data1$trt)
      lrpval0.2<-onesidep.survdiff(data1$dcoFA0time,data1$dcoFA0ind,data1$trt)
      lrpval1.1<-onesidep.survdiff(data1$dcoIA1time,data1$dcoIA1ind,data1$trt)
      lrpval1.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA1ind,data1$trt)
      lrpval2.1<-onesidep.survdiff(data1_mITT$dcoIA1time,data1_mITT$dcoIA1ind,data1_mITT$trt)
      lrpval2.2<-onesidep.survdiff(data1_mITT$dcoFA1time,data1_mITT$dcoFA1ind,data1_mITT$trt)
      lrpval3.1<-onesidep.survdiff(data1$dcoIA2time,data1$dcoIA2ind,data1$trt)
      lrpval3.2<-onesidep.survdiff(data1$dcoFA2time,data1$dcoFA2ind,data1$trt)      
      lrpval4.1<-onesidep.survdiff(data1$dcoIA1time,data1$dcoIA3ind,data1$trt)
      lrpval4.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA3ind,data1$trt)
      
      csp5.1<-pchisq(summary(os_cp5.1)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.1 <- ifelse( os_cp5.1$coef <= 0, csp5.1/2, 1 - csp5.1/2)  
      csp5.2<-pchisq(summary(os_cp5.2)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.2 <- ifelse( os_cp5.2$coef <= 0, csp5.2/2, 1 - csp5.2/2)  
      
      ind0.1[i]<-as.numeric(lrpval0.1<alphaIA)
      ind0.2[i]<-as.numeric(lrpval0.2<alphaFA)
      ind1.1[i]<-as.numeric(lrpval1.1<alphaIA)
      ind1.2[i]<-as.numeric(lrpval1.2<alphaFA)
      ind2.1[i]<-as.numeric(lrpval2.1<alphaIA)
      ind2.2[i]<-as.numeric(lrpval2.2<alphaFA)
      ind3.1[i]<-as.numeric(lrpval3.1<alphaIA)
      ind3.2[i]<-as.numeric(lrpval3.2<alphaFA)
      ind4.1[i]<-as.numeric(lrpval4.1<alphaIA)
      ind4.2[i]<-as.numeric(lrpval4.2<alphaFA)
      ind5.1[i]<-as.numeric(plrtpval5.1<alphaIA)
      ind5.2[i]<-as.numeric(plrtpval5.2<alphaFA)      
      
      logHR0.1[i]<-summary(os_cp0.1)$coefficients[1]
      logHR0.2[i]<-summary(os_cp0.2)$coefficients[1]
      logHR1.1[i]<-summary(os_cp1.1)$coefficients[1]
      logHR1.2[i]<-summary(os_cp1.2)$coefficients[1]
      logHR2.1[i]<-summary(os_cp2.1)$coefficients[1]
      logHR2.2[i]<-summary(os_cp2.2)$coefficients[1]
      logHR3.1[i]<-summary(os_cp3.1)$coefficients[1]
      logHR3.2[i]<-summary(os_cp3.2)$coefficients[1]
      logHR4.1[i]<-summary(os_cp4.1)$coefficients[1]
      logHR4.2[i]<-summary(os_cp4.2)$coefficients[1]
      logHR5.1[i]<-summary(os_cp5.1)$coef[1]
      logHR5.2[i]<-summary(os_cp5.2)$coef[1]
      
      time.0[i]<-sort(data1[data1$dcoFA0ind==1,]$dcoFA0caltime)[FAevent] #No COVID19, TIME for FA
      time.1[i]<-sort(data1[data1$dcoFA1ind==1,]$dcoFA1caltime)[FAevent] #COVID19, TIME for FA
      time.2[i]<-sort(data1[data1$dcoFA2ind==1,]$dcoFA2caltime)[FAevent] #COVID19, TIME for FA with censoring approach
      if(i %in% seq(1000,simu,1000)) cat(i)
    }
    power0.1<-mean(ind0.1);power0.2<-mean(ind0.2);power1.1<-mean(ind1.1);power1.2<-mean(ind1.2);
    power2.1<-mean(ind2.1);power2.2<-mean(ind2.2);power3.1<-mean(ind3.1);power3.2<-mean(ind3.2);
    power4.1<-mean(ind4.1);power4.2<-mean(ind4.2);power5.1<-mean(ind5.1);power5.2<-mean(ind5.2);
    
    mHR0.1<-exp(mean(logHR0.1));mHR0.2<-exp(mean(logHR0.2));mHR1.1<-exp(mean(logHR1.1));mHR1.2<-exp(mean(logHR1.2));
    mHR2.1<-exp(mean(logHR2.1));mHR2.2<-exp(mean(logHR2.2));mHR3.1<-exp(mean(logHR3.1));mHR3.2<-exp(mean(logHR3.2));
    mHR4.1<-exp(mean(logHR4.1));mHR4.2<-exp(mean(logHR4.2));mHR5.1<-exp(mean(logHR5.1));mHR5.2<-exp(mean(logHR5.2));
    
    crHR0.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR0.1)))
    crHR0.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR0.2)))
    crHR1.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR1.1)))
    crHR1.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR1.2)))
    crHR2.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR2.1)))
    crHR2.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR2.2)))
    crHR3.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR3.1)))
    crHR3.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR3.2)))
    crHR4.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR4.1)))
    crHR4.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR4.2)))
    crHR5.1<-exp(log(hr.cv.IA)-qnorm(1-alphaIA)*(sd(logHR5.1)))
    crHR5.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR5.2)))
    
    time0<-mean(time.0,na.rm=T)
    time1<-mean(time.1,na.rm=T)
    time2<-mean(time.2,na.rm=T)
    
    powerIA<-c(power0.1,power1.1,power2.1,power3.1,power4.1,power5.1)
    powerFA<-c(power0.2,power1.2,power2.2,power3.2,power4.2,power5.2)
    mHRIA  <-c(mHR0.1,mHR1.1,mHR2.1,mHR3.1,mHR4.1,mHR5.1)
    mHRFA  <-c(mHR0.2,mHR1.2,mHR2.2,mHR3.2,mHR4.2,mHR5.2)
    crHRIA <-c(crHR0.1,crHR1.1,crHR2.1,crHR3.1,crHR4.1,crHR5.1)
    crHRFA <-c(crHR0.2,crHR1.2,crHR2.2,crHR3.2,crHR4.2,crHR5.2)
    time <-c(time0,time1,time2)
    result<-list(round(powerIA,4),round(powerFA,4),round(mHRIA,4),round(mHRFA,4),round(crHRIA,4),round(crHRFA,4),round(time,4))
    names(result)<-c('PowerIA','PowerFA','HRIA','HRFA','CVHRIA','CVHRFA','Time')
  }
  
  else if (!includeIA & isOS){ # OS without IA
    for (i in 1:simu){
      data1<-as.data.frame(data[,,i])
      colnames(data1)<-c('SUBJID','trt','fultime','entryt','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                         'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
      data1_mITT <- subset(data1,covidind!=1)
      covmatrix<- matrix(as.numeric(data1$trt == 1));colnames(covmatrix) <- "trt" # covariate treated as matrix, required in crr.
      
      os_cp0.2<-coxph(Surv(dcoFA0time,dcoFA0ind)~trt,ties="efron" ,data=data1); #FA, No COVID
      os_cp1.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1); #FA, COVID ITT
      os_cp2.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1_mITT); #FA, COVID mITT
      os_cp3.2<-coxph(Surv(dcoFA2time,dcoFA2ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with updated DCO
      os_cp4.2<-coxph(Surv(dcoFA1time,dcoFA3ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with original DCO
      os_cp5.2<-crr(ftime = data1$dcoFA1time, fstatus = data1$dcoFA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #FA, Competing Risk
      
      lrpval0.2<-onesidep.survdiff(data1$dcoFA0time,data1$dcoFA0ind,data1$trt)
      lrpval1.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA1ind,data1$trt)
      lrpval2.2<-onesidep.survdiff(data1_mITT$dcoFA1time,data1_mITT$dcoFA1ind,data1_mITT$trt)
      lrpval3.2<-onesidep.survdiff(data1$dcoFA2time,data1$dcoFA2ind,data1$trt)      
      lrpval4.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA3ind,data1$trt)
      
      csp5.2<-pchisq(summary(os_cp5.2)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.2 <- ifelse( os_cp5.2$coef <= 0, csp5.2/2, 1 - csp5.2/2)  
      
      ind0.2[i]<-as.numeric(lrpval0.2<alphaFA)
      ind1.2[i]<-as.numeric(lrpval1.2<alphaFA)
      ind2.2[i]<-as.numeric(lrpval2.2<alphaFA)
      ind3.2[i]<-as.numeric(lrpval3.2<alphaFA)
      ind4.2[i]<-as.numeric(lrpval4.2<alphaFA)
      ind5.2[i]<-as.numeric(plrtpval5.2<alphaFA)      
      
      logHR0.2[i]<-summary(os_cp0.2)$coefficients[1]
      logHR1.2[i]<-summary(os_cp1.2)$coefficients[1]
      logHR2.2[i]<-summary(os_cp2.2)$coefficients[1]
      logHR3.2[i]<-summary(os_cp3.2)$coefficients[1]
      logHR4.2[i]<-summary(os_cp4.2)$coefficients[1]
      logHR5.2[i]<-summary(os_cp5.2)$coef[1]
      
      time.0[i]<-sort(data1[data1$dcoFA0ind==1,]$dcoFA0caltime)[FAevent] #No COVID19, TIME for FA
      time.1[i]<-sort(data1[data1$dcoFA1ind==1,]$dcoFA1caltime)[FAevent] #COVID19, TIME for FA
      time.2[i]<-sort(data1[data1$dcoFA2ind==1,]$dcoFA2caltime)[FAevent] #COVID19, TIME for FA with censoring approach
      if(i %in% seq(1000,simu,1000)) cat(i)
    }
    power0.2<-mean(ind0.2);power1.2<-mean(ind1.2);power2.2<-mean(ind2.2);power3.2<-mean(ind3.2);power4.2<-mean(ind4.2);power5.2<-mean(ind5.2);
    mHR0.2<-exp(mean(logHR0.2));mHR1.2<-exp(mean(logHR1.2));mHR2.2<-exp(mean(logHR2.2));mHR3.2<-exp(mean(logHR3.2));mHR4.2<-exp(mean(logHR4.2));mHR5.2<-exp(mean(logHR5.2));
    
    crHR0.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR0.2)))
    crHR1.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR1.2)))
    crHR2.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR2.2)))
    crHR3.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR3.2)))
    crHR4.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR4.2)))
    crHR5.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR5.2)))
    
    time0<-mean(time.0,na.rm=T)
    time1<-mean(time.1,na.rm=T)
    time2<-mean(time.2,na.rm=T)
    
    powerFA<-c(power0.2,power1.2,power2.2,power3.2,power4.2,power5.2)
    mHRFA  <-c(mHR0.2,mHR1.2,mHR2.2,mHR3.2,mHR4.2,mHR5.2)
    crHRFA <-c(crHR0.2,crHR1.2,crHR2.2,crHR3.2,crHR4.2,crHR5.2)
    time <-c(time0,time1,time2)
    result<-list(round(powerFA,4),round(mHRFA,4),round(crHRFA,4),round(time,4))
    names(result)<-c('PowerFA','HRFA','CVHRFA','Time')
    
  }
  
  else { # PFS without IA
    for (i in 1:simu){
      data1<-as.data.frame(data[,,i])
      colnames(data1)<-c('SUBJID','trt','fultime','entryt','pfs','caltime0','caltime1','covidind','dcoIA0caltime','dcoIA0time','dcoIA0ind','dcoFA0caltime','dcoFA0time','dcoFA0ind','dcoIA1caltime','dcoIA1time','dcoIA1ind',
                         'dcoFA1caltime','dcoFA1time','dcoFA1ind','dcoIA2caltime','dcoIA2time','dcoIA2ind','dcoFA2caltime','dcoFA2time','dcoFA2ind','dcoIA3ind','dcoFA3ind','dcoIA4ind','dcoFA4ind')
      data1_mITT <- subset(data1,covidind!=1)
      covmatrix<- matrix(as.numeric(data1$trt == 1));colnames(covmatrix) <- "trt" # covariate treated as matrix, required in crr.
      
      os_cp0.2<-coxph(Surv(dcoFA0time,dcoFA0ind)~trt,ties="efron" ,data=data1); #FA, No COVID
      os_cp1.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1); #FA, COVID ITT
      os_cp2.2<-coxph(Surv(dcoFA1time,dcoFA1ind)~trt,ties="efron" ,data=data1_mITT); #FA, COVID mITT
      os_cp3.2<-coxph(Surv(dcoFA2time,dcoFA2ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with updated DCO
      os_cp4.2<-coxph(Surv(dcoFA1time,dcoFA3ind)~trt,ties="efron" ,data=data1); #FA, Censor COVID with original DCO
      os_cp5.2<-crr(ftime = data1$dcoFA1time, fstatus = data1$dcoFA4ind, cov1=covmatrix,failcode= 1,cencode = 0); #FA, Competing Risk
      
      lrpval0.2<-onesidep.survdiff(data1$dcoFA0time,data1$dcoFA0ind,data1$trt)
      lrpval1.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA1ind,data1$trt)
      lrpval2.2<-onesidep.survdiff(data1_mITT$dcoFA1time,data1_mITT$dcoFA1ind,data1_mITT$trt)
      lrpval3.2<-onesidep.survdiff(data1$dcoFA2time,data1$dcoFA2ind,data1$trt)      
      lrpval4.2<-onesidep.survdiff(data1$dcoFA1time,data1$dcoFA3ind,data1$trt)
      
      csp5.2<-pchisq(summary(os_cp5.2)$logtest,length(unique(data1$trt)) - 1,lower.tail=F); plrtpval5.2 <- ifelse( os_cp5.2$coef <= 0, csp5.2/2, 1 - csp5.2/2)  
      
      ind0.2[i]<-as.numeric(lrpval0.2<alphaFA)
      ind1.2[i]<-as.numeric(lrpval1.2<alphaFA)
      ind2.2[i]<-as.numeric(lrpval2.2<alphaFA)
      ind3.2[i]<-as.numeric(lrpval3.2<alphaFA)
      ind4.2[i]<-as.numeric(lrpval4.2<alphaFA)
      ind5.2[i]<-as.numeric(plrtpval5.2<alphaFA)      
      
      logHR0.2[i]<-summary(os_cp0.2)$coefficients[1]
      logHR1.2[i]<-summary(os_cp1.2)$coefficients[1]
      logHR2.2[i]<-summary(os_cp2.2)$coefficients[1]
      logHR3.2[i]<-summary(os_cp3.2)$coefficients[1]
      logHR4.2[i]<-summary(os_cp4.2)$coefficients[1]
      logHR5.2[i]<-summary(os_cp5.2)$coef[1]
      
      time.0[i]<-sort(data1[data1$dcoFA0ind==1,]$dcoFA0caltime)[FAevent] #No COVID19, TIME for FA
      time.1[i]<-sort(data1[data1$dcoFA1ind==1,]$dcoFA1caltime)[FAevent] #COVID19, TIME for FA
      time.2[i]<-sort(data1[data1$dcoFA2ind==1,]$dcoFA2caltime)[FAevent] #COVID19, TIME for FA with censoring approach
      if(i %in% seq(1000,simu,1000)) cat(i)
    }
    power0.2<-mean(ind0.2);power1.2<-mean(ind1.2);power2.2<-mean(ind2.2);power3.2<-mean(ind3.2);power4.2<-mean(ind4.2);power5.2<-mean(ind5.2);
    mHR0.2<-exp(mean(logHR0.2));mHR1.2<-exp(mean(logHR1.2));mHR2.2<-exp(mean(logHR2.2));mHR3.2<-exp(mean(logHR3.2));mHR4.2<-exp(mean(logHR4.2));mHR5.2<-exp(mean(logHR5.2));
    
    crHR0.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR0.2)))
    crHR1.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR1.2)))
    crHR2.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR2.2)))
    crHR3.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR3.2)))
    crHR4.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR4.2)))
    crHR5.2<-exp(log(hr.cv.FA)-qnorm(1-alphaFA)*(sd(logHR5.2)))
    
    time0<-mean(time.0,na.rm=T)
    time1<-mean(time.1,na.rm=T)
    time2<-mean(time.2,na.rm=T)
    
    powerFA<-c(power0.2,power1.2,power2.2,power3.2,power4.2,power5.2)
    mHRFA  <-c(mHR0.2,mHR1.2,mHR2.2,mHR3.2,mHR4.2,mHR5.2)
    crHRFA <-c(crHR0.2,crHR1.2,crHR2.2,crHR3.2,crHR4.2,crHR5.2)
    time <-c(time0,time1,time2)
    result<-list(round(powerFA,4),round(mHRFA,4),round(crHRFA,4),round(time,4))
    names(result)<-c('PowerFA','HRFA','CVHRFA','Time')
  }    
  return(result)  
}

T1Eplot<-function(PowerFA,filename,ylim,alpha,legendpos){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(PowerFA[1,])
  plot(PowerFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  lines(PowerFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  lines(PowerFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  lines(PowerFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  lines(PowerFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  lines(PowerFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Type I Error',xlab='COVID Related Death')
  abline(h=alpha,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=1:nCOVID,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}

Powerplot<-function(PowerFA,filename,ylim,power,legendpos){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(PowerFA[1,])
  plot(PowerFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Power',xlab='COVID Related Death')
  lines(PowerFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Power',xlab='COVID Related Death')
  lines(PowerFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Power',xlab='COVID Related Death')
  lines(PowerFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Power',xlab='COVID Related Death')
  lines(PowerFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Power',xlab='COVID Related Death')
  lines(PowerFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Power',xlab='COVID Related Death')
  abline(h=power,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=1:nCOVID,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}



HRplot<-function(HRFA,filename,ylim,truth,legendpos){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(HRFA[1,])
  plot(HRFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  lines(HRFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  lines(HRFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  lines(HRFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  lines(HRFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  lines(HRFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Hazard Ratio',xlab='COVID Related Death')
  abline(h=truth,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=1:nCOVID,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}

TIMET1Eplot<-function(PowerFA,filename,ylim,alpha,legendpos,time){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(PowerFA[1,])
  plot(PowerFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Type I Error',xlab='Pandemic Time Prior to FA')
  abline(h=alpha,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=time,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}

TIMEHRplot<-function(HRFA,filename,ylim,truth,legendpos,time){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(HRFA[1,])
  plot(HRFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  lines(HRFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  lines(HRFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  lines(HRFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  lines(HRFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  lines(HRFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Hazard Ratio',xlab='Pandemic Time Prior to FA')
  abline(h=truth,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=time,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}

TIMEPowerplot<-function(PowerFA,filename,ylim,power,legendpos,time){
  jpeg(file=paste0(filename,".jpeg"),width=3200,height=2400,res=300)
  nCOVID<-length(PowerFA[1,])
  plot(PowerFA[1,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[1],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[2,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[2],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[3,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[3],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[4,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[4],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[5,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[5],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  lines(PowerFA[6,],type='l',lwd=2,xlim=c(0,nCOVID+1),ylim=ylim,col=cols[6],xaxt='n',ylab='Power',xlab='Pandemic Time Prior to FA')
  abline(h=power,lty=2,col='red')
  axis(1, at=1:nCOVID,labels=time,las=0, cex.axis=1, tck=-.01)
  legend(legendpos,
         c('ITT  (NO PANDEMIC)',
           'ITT  (PANDEMIC)',
           'mITT(PANDEMIC)',
           'CP   (PANDEMIC)',
           'CS   (PANDEMIC)',
           'CR   (PANDEMIC)'),
         text.col=c(cols[1:6]),bty='n', cex=1.2)
  grid()
  dev.off()
}
